/*
    KWin - the KDE window manager
    This file is part of the KDE project.

    SPDX-FileCopyrightText: 2013 Martin Gräßlin <mgraesslin@kde.org>

    SPDX-License-Identifier: GPL-2.0-or-later
*/
#include <kwineffects.h>
Q_LOGGING_CATEGORY(KWINEFFECTS, "kwineffects", QtCriticalMsg)
